Package: fSeries
Version: 240.10068
Date: 1996 - 2006
Title: Rmetrics - The Dynamical Process Behind Markets
Author: Diethelm Wuertz and many others, see the SOURCE file
Depends: R (>= 1.9.0), methods, mgcv, nnet, fCalendar, fBasics
Maintainer: Diethelm Wuertz and Rmetrics Core Team <wuertz@itp.phys.ethz.ch>
Description: Environment for teaching "Financial Engineering and Computational Finance"
License: GPL Version 2 or later
URL: http://www.rmetrics.org 
Packaged: Sun Oct 22 13:09:12 2006; myself
